/*
  XVarStar, GCVS catalogue explorer
  Copyright (C) 2001-2008 Ivan Vitjuk 

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#include "variable.h"

struct ConfigData config_data;

struct CfgTokens cfg_tokens[] = {
        {"component", OUT_COMPONENT},
        {"noteflag", OUT_NOTEFLAG},
        {"ra1950", OUT_RA1950},
        {"dec1950", OUT_DEC1950},
        {"u_des", OUT_U_DES},
        {"l_magmax", OUT_L_MAGMAX},
        {"u_magmax", OUT_U_MAGMAX},
        {"f_magmax", OUT_F_MAGMAX},
        {"l_magmin", OUT_L_MAGMIN},
        {"u_magmin", OUT_U_MAGMIN},
        {"n_magmin", OUT_N_MAGMIN},
        {"f_magmin", OUT_F_MAGMIN},
        {"magcode", OUT_MAGCODE},
        {"epoch", OUT_EPOCH},
        {"q_epoch", OUT_Q_EPOCH},
        {"yearnova", OUT_YEARNOVA},
        {"l_period", OUT_L_PERIOD},
        {"period", OUT_PERIOD},
        {"u_period", OUT_U_PERIOD},
        {"n_period", OUT_N_PERIOD},
        {"mmd", OUT_MMD},
        {"u_mmd", OUT_U_MMD},
        {"n_mmd", OUT_N_MMD},
        {"ref1", OUT_REF1},
        {"ref2", OUT_REF2},
        {"exists", OUT_EXISTS},
        {NULL,  0}
};

static void remove_tailspaces(char *str)
{
        while(*str){
                if(*str == ' '){
                        *str = '\0';
                        return;
                }
                str++;
        }
}

static int configure_output(const char *str)
{
        char buf[32];
        int i;
        struct CfgTokens *tmptok;
        
        config_data.out_flags = 0;
        
        while(*str){
                for(i = 0; *str && *str != ',' && i < sizeof(buf)-1; i++, str++)
                        buf[i] = *str;
                if(*str)
                        str++;
                buf[i] = '\0';

                for(tmptok = cfg_tokens; tmptok->token; tmptok++)
                        if(strcasecmp(tmptok->token, buf) == 0)
                                break;
                
                if(!tmptok->token){
                        var_error_reason("Unknown output_flags token: %s", buf);
                        return 0;
                }

                config_data.out_flags |= tmptok->bit;
        }

        return 1;
}

static int do_config(const char *tok, const char *val, struct ConfigData *cfg)
{
        if(strcasecmp(tok, "$gcvs_database") == 0)
                cfg->gcvs_database = strdup(val);
        else if(strcasecmp(tok, "$output_flags") == 0)
                return configure_output(val);
        else{
                var_error_reason("Unknown token: %s", tok);
                return 0;
        }
        return 1;
}

static int _var_parse_line(const char *line, struct ConfigData *cfg)
{
        char *ptr, token[32], value[128];
        int len;
 
        // skip comment and empty lines
        if(*line == '#' || *line == '\n' || !*line)
                return 1;

        // '$' must be first character
        if(*line != '$'){
                var_error_reason("Line doesn't begin with '$'");
                return 0;
        }

        // get config token and remove spaces from tail
        ptr = strchr(line, '=');        
        len = ptr - line;
        strncpy(token, line, MAX(len, sizeof(token)));
        token[sizeof(token)-1] = 0;
        remove_tailspaces(token);
        
        // skip spaces after '='
        for(ptr++; *ptr && *ptr == ' '; ptr++);
        
        // get valuse and remove spaces from tail        
        strncpy(value, ptr, sizeof(value));
        value[sizeof(value)-1] = '\0';
        remove_tailspaces(value);
        
        return do_config(token, value, cfg);
}

char *var_get_cfgfilename()
{
        return XVS_CONF;
}

int var_configure()
{
        FILE *fp;
        char buf[256], *filename;
        
        if(!(filename = var_get_cfgfilename())){
                var_set_error(10);
                return 0;
        }

        if(!(fp = fopen(filename, "r"))){
                var_set_error(5);
                var_error_reason("fopen: %s: %s", filename, strerror(errno));
                return 0;
        }
                
        while(fgets(buf, sizeof(buf), fp)){
                buf[sizeof(buf)-1] = '\0';
                buf[strlen(buf)-1] = '\0';
                if(!_var_parse_line(buf, &config_data)){
                        var_set_error(5);
                        return 0;
                }
        }
        return 1;
}
